%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [CompC, numComps] = CosupportComponents(OmegaL)

n = size(OmegaL, 1);
n = n/2;

OLv = OmegaL(1:n, :);
OLh = OmegaL(n+1:2*n, :)';

numComps = 0;
CompC = zeros(n, n);

processingQx = zeros(n^2, 1);
processingQy = zeros(n^2, 1);
pQI = 0;

ii = 1;
jj = 1;
while true
  while pQI > 0
    ti = processingQx(pQI);
    tj = processingQy(pQI);
    if ti < n && OLh(ti, tj) == 1
      CompC(ti+1, tj) = numComps;
      pQI = pQI + 1;
      processingQx(pQI) = ti + 1;
      processingQy(pQI) = tj;
      OLh(ti, tj) = 0;
    elseif ti > 1 && OLh(ti-1, tj) == 1
      CompC(ti-1, tj) = numComps;
      pQI = pQI + 1;
      processingQx(pQI) = ti - 1;
      processingQy(pQI) = tj;
      OLh(ti-1, tj) = 0;
    elseif tj < n && OLv(ti, tj) == 1
      CompC(ti, tj+1) = numComps;
      pQI = pQI + 1;
      processingQx(pQI) = ti;
      processingQy(pQI) = tj+1;
      OLv(ti, tj) = 0;
    elseif tj > 1 && OLv(ti, tj-1) == 1
      CompC(ti, tj-1) = numComps;
      pQI = pQI + 1;
      processingQx(pQI) = ti;
      processingQy(pQI) = tj - 1;
      OLv(ti, tj-1) = 0;
    else
      %disp(['----', num2str(ti), ' ', num2str(tj)]);
      pQI = pQI - 1;
    end
  end
  
  if ii < n && OLh(ii, jj) == 1
    numComps = numComps + 1;
    CompC(ii, jj) = numComps;
    CompC(ii+1, jj) = numComps;
    pQI = pQI + 1;
    processingQx(pQI) = ii;
    processingQy(pQI) = jj;
    pQI = pQI + 1;
    processingQx(pQI) = ii+1;
    processingQy(pQI) = jj;
    OLh(ii, jj) = 0;
  end
  if pQI == 0 && jj < n && OLv(ii, jj) == 1
    numComps = numComps + 1;
    CompC(ii, jj) = numComps;
    CompC(ii, jj+1) = numComps;
    pQI = pQI + 1;
    processingQx(pQI) = ii;
    processingQy(pQI) = jj;
    pQI = pQI + 1;
    processingQx(pQI) = ii;
    processingQy(pQI) = jj+1;
    OLv(ii, jj) = 0;
  end
  
  if jj < n
    jj = jj + 1;
  elseif ii < n
    ii = ii + 1;
    jj = 1;
  else
    break;
  end
end

numComps