%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [v, snr, y] = MtMn(X, Loc, noise_level)

% returns v = M^* (M X + z)
% where M is a sensing matrix M for n x n image
% corresponding to sampling Fourier coefficients
% at locations specified by Loc
% z is a random gaussian with variance noise_level * norm(MX)

n = size(X, 1);
v = zeros(n, n);
z = zeros(n, n);
w = fft2(X);
Lt = logical(Loc);
v(Lt) = w(Lt);
zt = randn(n, n);
z(Lt) = zt(Lt);
z = noise_level * norm(v, 'fro') *z / norm(z,'fro');
snr = norm(v, 'fro') / norm(z, 'fro');
y = v + z;
v = ifft2(y);
y = y(Lt);
