%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function c = Omega(X)

% returns c = Omega * X
% where Omega corresponds to all the one-step differences
% in an n x n image.
% X must then be an n x n image.
% Omega is in fact represented by (2*n) x (n-1) array
% with the following convention:
% if Omega(i, j) = 1, then it corresponds to difference
% X(i, j+1) - X(i, j),
% while if Omega(n+i, j) = 1, then it corresponds to
% X(j+1, i) - X(j, i).
% Otherwise, Omega(i, j) should be 0.

n = size(X, 1);
v = X(:, 2:end) - X(:, 1:(end-1));
h = X(2:end, :) - X(1:(end-1), :);
c = [v; h.'];
%c = [v; h.'] .* Omega;
