%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xhat, Lambdahat] = RAHTP(y, M, MH, Omega, OmegaH, params, xinit)

d = length(xinit(:));

if strcmp(class(Omega), 'function_handle')
    p = length(Omega(zeros(d,1)));
else    %% Omega is a matrix
    p = size(Omega, 1);
end

iter = 0;
lagmult = 1e-4;
Lambdahat = 1:p;
y_res = y;

options.Method = 'lbfgs';
options.Corr = 500;

while iter < params.num_iteration
    iter = iter + 1;
    descent = MH(y - M(xinit));
    LambdaEst = FindRowsToRemove(Omega(xinit), params.cosparsity_level);
    mu = gradient_descent_step(xinit, descent, y, M, Omega, OmegaH, LambdaEst, params);
    if isnan(mu)
        mu = 0;
    end
    xhat_grad = xinit + mu * descent;
    disp(['fidelity_error=', num2str(norm(M(xhat_grad)-y))]);
    Omega_xhat_grad = Omega(xhat_grad);
    Lambdahat = FindRowsToRemove(Omega_xhat_grad, params.cosparsity_level);
    maxcoef = max(Omega_xhat_grad(Lambdahat));
    TheHermitianMatrix_op = @(b) TheHermitianMatrix(b, M, MH, Omega, OmegaH, Lambdahat, params.lm);
    xhat = CGLS_solve(MH(y), TheHermitianMatrix_op, xhat_grad, params);
    xhat = real(xhat);
    %TheHermitianMatrix_quad_func = @(b) TheHermitianMatrix_quad(b, MH(y), M, MH, Omega, OmegaH, Lambdahat, params.lm);
    %xhat = minFunc(TheHermitianMatrix_quad_func, xhat_grad,options);
    %post process
    Omega_xhat = Omega(xhat);
    Lambdahat = FindRowsToRemove(Omega_xhat,  params.cosparsity_level);
    xhat = cosparse_projection(xhat, Omega, OmegaH, Lambdahat, params);
    %end post process
    y_res_norm = norm(y- M(xhat));
    cur_cosparsity = sum(abs(Omega(xhat))<1e-3);
    disp(['**AHTP iter = ',num2str(iter) ,'  maxcoef=', num2str(maxcoef), ' ||y_res||_2=', num2str(y_res_norm), ' current cosparsity =',num2str(cur_cosparsity)]);
    %disp([' target= ', num2str(params.stopping_relative_solution_change), ' relative_solution_change =', num2str(norm(xhat-xinit)/norm(xhat))]);
    if check_stopping_criteria(xhat, xinit, maxcoef, y_res_norm, params)
        break;
    end
    xinit = xhat;
       

    
end
return;


function mu = gradient_descent_step(xprev, descent, y, M, Omega, OmegaH, LambdaEst, params)

%options.Method = 'lbfgs';
%options.Corr = 1000;
%options.MaxIter = 500;
%options.useComplex = 1;
%params.max_inner_iteration = 500;

[~, tempind] = sort(abs(Omega(descent)));
DescentLambdaEst = sort(tempind(1:params.cosparsity_level));
IntersectLambdaEst = intersect(DescentLambdaEst, LambdaEst);
descent_proj = cosparse_projection(descent, Omega, OmegaH, IntersectLambdaEst, params);
mu = norm(descent_proj, 2)^2 / norm(M(descent_proj), 2)^2;
return;


function [to_be_removed] = FindRowsToRemove(analysis_repr, target_l)

    abscoef = abs(analysis_repr(:));
    to_be_removed = find(abscoef < 1e-5);
    if (length(to_be_removed) > target_l)
        return;
    end
    [~, tempind] = sort(abscoef);
    to_be_removed = sort(tempind(1:target_l));
    return;

function r = check_stopping_criteria(xhat, xinit, maxcoef, y_res_norm, params)

    r = 0;

    if isfield(params, 'stopping_coefficient_size') && maxcoef < params.stopping_coefficient_size
        r = 1;
        return;
    end


    if isfield(params, 'stopping_relative_solution_change') && norm(xhat-xinit)/norm(xhat) < params.stopping_relative_solution_change
        r = 1;
        return;
    end

    if isfield(params, 'stopping_residual_size') && y_res_norm < params.stopping_residual_size
        r = 1;
        return;
    end