%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xhat, Lambdahat] = RASP(y, M, MH, Omega, OmegaH, params, xinit)

d = length(xinit(:));

if strcmp(class(Omega), 'function_handle')
    p = length(Omega(zeros(d,1)));
else    %% Omega is a matrix
    p = size(Omega, 1);
end

iter = 0;
lagmult = 1e-4;
Lambdahat = 1:p;
y_res = y;

while iter < params.num_iteration
    iter = iter + 1;
    target_l = ceil(params.alpha*params.cosparsity_level);
    [LambdaDelt] = FindRowsToRemove(Omega(MH(y_res)), target_l);
    LambdaTilde = intersect(LambdaDelt, Lambdahat);
    TheHermitianMatrix_op = @(b) TheHermitianMatrix(b, M, MH, Omega, OmegaH, LambdaTilde, params.lm);
    x_p = CGLS_solve(MH(y), TheHermitianMatrix_op, xinit, params);
    x_p = real(x_p);
    %post process
    %LambdaTilde = FindRowsToRemove(Omega(x_p),  length(LambdaTilde));
    %x_p = cosparse_projection(x_p, Omega, OmegaH, LambdaTilde, params);
    %end post process
    disp(['fidelity_error=', num2str(norm(M(x_p)-y))]);
    Omega_x_p = Omega(x_p);
    Lambdahat = FindRowsToRemove(Omega_x_p, params.cosparsity_level);
    maxcoef = max(Omega_x_p(Lambdahat));
    TheHermitianMatrix_op = @(b) TheHermitianMatrix(b, M, MH, Omega, OmegaH, Lambdahat, params.lm);
    xhat = CGLS_solve(MH(y), TheHermitianMatrix_op, x_p, params);
    xhat = real(xhat);
    %post process
    %Lambdahat = FindRowsToRemove(Omega(xhat),  params.cosparsity_level);
    %xhat = cosparse_projection(xhat, Omega, OmegaH, Lambdahat, params);
    %end post process
    %Lambdahat = find(abs(Omega(xhat))<1e-3);
    %cur_cosparsity = length(Lambdahat);
    cur_cosparsity = sum(abs(Omega(xhat))<1e-3);
    y_res = y- M(xhat);
    y_res_norm = norm(y_res);
    disp(['**ASP iter = ',num2str(iter) ,' maxcoef=', num2str(maxcoef), ' ||y_res||_2=', num2str(y_res_norm), ' current cosparsity =',num2str(cur_cosparsity)]);
    disp([' target= ', num2str(params.stopping_relative_solution_change), ' relative_solution_change =', num2str(norm(xhat-xinit)/norm(xhat))]);
    if check_stopping_criteria(xhat, xinit, maxcoef, y_res_norm, params)
        break;
    end
    xinit = xhat;

    %n = sqrt(d);
    %figure(9);
    %RR = zeros(2*n, n-1);
    %RR(Lambdahat) = 1;
    %XD = ones(n, n);
    %XD(:, 2:end) = XD(:, 2:end) .* RR(1:n, :);
    %XD(:, 1:(end-1)) = XD(:, 1:(end-1)) .* RR(1:n, :);
    %XD(2:end, :) = XD(2:end, :) .* RR((n+1):(2*n), :)';
    %XD(1:(end-1), :) = XD(1:(end-1), :) .* RR((n+1):(2*n), :)';
    %XD = FD2DiagnosisPlot(n, Lambdahat);
    %imshow(XD);
    %figure(10);
    %imshow(reshape(real(xhat), n, n));
end

return;



function [to_be_removed] = FindRowsToRemove(analysis_repr, target_l)

    abscoef = abs(analysis_repr(:));
    [~, tempind] = sort(abscoef);
    to_be_removed = sort(tempind(1:target_l));
    return;


function r = check_stopping_criteria(xhat, xinit, maxcoef, y_res_norm, params)

    r = 0;

    if isfield(params, 'stopping_coefficient_size') && maxcoef < params.stopping_coefficient_size
        r = 1;
        return;
    end


    if isfield(params, 'stopping_relative_solution_change') && norm(xhat-xinit)/norm(xhat) < params.stopping_relative_solution_change
        r = 1;
        return;
    end

    if isfield(params, 'stopping_residual_size') && y_res_norm < params.stopping_residual_size
        r = 1;
        return;
    end