%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all;
side_length_of_image = 256;
num_lines = 35;
noise_level = 0;
sample_type = 'full';        % values: l1magic, full
X = phantom('Modified Shepp-Logan', side_length_of_image);

figure();
imshow(X);
title(['original image' num2str(size(X,1)) 'x' num2str(size(X,2)) '=' num2str(numel(X))]);
hgradient = X(:,2:end)-X(:,1:(end-1));
vgradient = X(2:end,:)-X(1:(end-1),:);
gradient = [hgradient; vgradient'];
numborders = length(find(gradient));
figure();
imshow(abs(gradient)); 
title(['original image gradient: ' num2str(numborders) ' nonzeros']);
cosparsity = sum(sum(abs(gradient) < 1e-5));

n = size(X, 1);
if strcmp(sample_type, 'full')
Loc = SamplingLocations(n, num_lines);
elseif strcmp(sample_type, 'l1magic')
Loc = SamplingLocationsL1Magic(n, num_lines);
else
disp('unknown sample_type');
exit(0);
end

[b, snr,y] = MtMn(X, Loc, noise_level);
disp(['signal to noise ratio for the experiment=', num2str(snr)]);

figure();
imshow(fftshift(Loc));
title(['sample locations: ' num2str(length(find(Loc~=0)))]);
Loc = logical(Loc);
%% prms are parameters for algorithm
M = @(t) FourierSampling(t, n, Loc);
MH = @(u) FourierSamplingH(u, n, Loc);
Omega = @(t) FDAnalysis(t, n);
OmegaH = @(u) FDSynthesis(u, n);
params.num_iteration = 60;
params.cosparsity_level = cosparsity;
%params.greedy_level = 50;
%params.stopping_coefficient_size = 1e-5;
%params.stopping_relative_solution_change = 1e-5;
params.stopping_residual_size = norm(M(X) - M(b),'fro') + 1e-2; %For AHTP
params.l2solver = 'cg';   % in fact, 'pseudoinverse' does not even make sense.
params.max_inner_iteration = 500;
params.l2_accuracy = 1e-9;
params.noise_level = noise_level;
params.alpha = 1;
params.lm = 1;

X_rep = Omega(X);
X_rep = reshape(X_rep,2*n, n-1);
Xhat = zeros(n, n);   %% initial estimate
tstart = tic;
[xhat, Lambdahat] = RAHTP(y, M, MH, Omega, OmegaH, params, Xhat(:));
time_ran = toc(tstart);

SNR = 20 * log10(1/sqrt(mean(mean(abs(X-reshape(max(min(xhat,1),0),n,n)).^2))));
disp(['SNR=', num2str(SNR)]);

imagesc([real(reshape(max(min(xhat,1),0),256,256)), imag(reshape(max(min(xhat,1),0),256,256))]);
colormap(gray(256));