%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function x_proj = cosparse_projection(x, Omega, OmegaH, Lambda, params)
% x is the vector to be projected
% Lambda is the cossuport
% Omega and OmegaH are the operator Omega and its conjugate

if isfield(params,'Q')
   x_proj = params.Q*x;
   return;
end

if strcmp(class(Omega),'function_handle')
    Omega_x = Omega(x);
    Omega_x_Lambda = zeros(size(Omega_x));
    Omega_x_Lambda(Lambda) = Omega_x(Lambda);
    OmegaLHOmegaL_op = @(b) OmegaLHOmegaL(b, Omega, OmegaH, Lambda);
    x_p_proj = CGLS_solve(OmegaH(Omega_x_Lambda), OmegaLHOmegaL_op, zeros(size(x)), params);
    x_proj = x - x_p_proj;
else 
   OmL = Omega(Lambda,:);
   x_proj_inv = (OmL*OmL') \ (OmL*x);
   x_proj = x - OmL'*x_proj_inv;
   %x_proj = x - pinv(OmL)*OmL*x;
end
