%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xhat, LambdaEst,C, C_tilde, C2] = ACoSaMP(y,Omega,M,l,alpha, x0, stats)

p = size(Omega,1);
m = size(M,1);
d = size(M,2);

max_iter = 500;
convThresh = 1e-6;

if nargin < 5
    alpha = 1;
end
C = 0;
C_tilde = inf;
C2 = 0;
if nargin > 5
   Lambda0 = find(abs(Omega*x0) < 1e-6);
end
xprev = zeros(d, 1);
LambdaEst = 1:p;
y_res = y;
MM = M'*M;
for j = 1:max_iter
    if nargin > 6
        disp('*************')
        disp(['iteration ', num2str(j)]);
    end
    [~, tempind] = sort(abs(Omega*M'*y_res));
    target_l = ceil(alpha*l);
    LambdaDelt = sort(tempind(1:target_l));
    LambdaTilde = intersect(LambdaDelt, LambdaEst);
    if nargin > 5
        Myres = M'*y_res;
        Myres_proj =  Myres- pinv(Omega(LambdaTilde,:)) * Omega(LambdaTilde,:) * Myres;
        LambdaEst0 = intersect(Lambda0, LambdaEst);
        Myres_proj0 =  Myres- pinv(Omega(LambdaEst0,:)) * Omega(LambdaEst0,:) * Myres;
        C2_res = norm(Myres - Myres_proj).^2/norm(Myres - Myres_proj0).^2;
        C_tilde_res = norm(Myres_proj).^2/norm(Myres_proj0).^2;
        C2 = max(C2, C2_res);
        C_tilde = min(C_tilde, C_tilde_res);
    end
    OmL = Omega(LambdaTilde,:);
    %OmLPerp = eye(d) - pinv(OmL)*OmL; the following condition implements this projection.
    if (size(OmL,1) < size(OmL,2))
        OmLPerp = eye(d) - OmL'*pinv(OmL*OmL')*OmL;
    else
        OmLPerp = eye(d) - pinv(OmL'*OmL)*(OmL'*OmL);
    end
    x_p = pinv( [OmLPerp*MM; OmL])*[OmLPerp*M'*y; zeros(length(LambdaTilde),1)];
%    x_p = pinv( [M; Omega(LambdaTilde,:)])*[y; zeros(length(LambdaTilde),1)]; the wrong way to make the projection
    [~, tempind] = sort(abs(Omega* x_p));
    LambdaEst = sort(tempind(1:l));
    xhat = x_p- pinv(Omega(LambdaEst,:)) * Omega(LambdaEst,:) * x_p;
    y_res = y - M*xhat;
    if nargin > 6
        disp(['rel_err = ', num2str(norm(xhat-x0)/norm(x0))]);
        disp(['delta = ', num2str(norm(xhat-xprev)/norm(xhat))]);
        disp(['restricted isometry value = ', num2str((norm(M*(xhat-xprev))/norm(xhat-xprev))^2)]);
    end
    if nargin > 5
       new_C = norm(xhat - x_p).^2/norm(x0 - x_p).^2;
       C = max(C, new_C); 
    end
    if norm(xhat-xprev)/norm(xhat) < convThresh || (norm(xhat) < convThresh)
        break;
    end
    xprev = xhat;
end
temp = zeros(p,1);
temp(LambdaEst) = 1;
LambdaEst =temp;
return;

% function xhat = estimate(xin, OmL, M, y)
% xhat = xin - pinv(OmL) * OmL * xin;
% %[nr, nc] = size(OmL);
% %OmLPerp = eye(nc) - pinv(OmL)*OmL;
% %xhat = pinv([OmLPerp*M'*M; OmL]) * [OmLPerp*M'*y; zeros(nr, 1)];
 
