%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if last parameter exists and contain x0 - the real origianl signal- 
% the user gets output from the algorithm while it is running.
%If last parameter exists and contain D then the final result is D*xhat. 
function [xhat, LambdaEst] = ASP(y,Omega,M,cosparsity,alpha, options)

p = size(Omega,1);
m = size(M,1);
d = size(M,2);

max_iter = 500;
convThresh = 1e-6;

if nargin < 5
    alpha = 1;
end
C = 0;
C_tilde = inf;
C2 = 0;
xprev = zeros(d, 1);
LambdaEst = 1:p;
y_res = y;
MM = M'*M;
l = cosparsity(1);
for j = 1:max_iter
    if nargin > 5 && isfield(options, 'x0')
        disp('*************')
        disp(['iteration ', num2str(j)]);
    end
    Myres = M'*y_res;
    LambdaDelt = find_rows_to_remove(Myres, Omega,  max(ceil(alpha*cosparsity),1));
    LambdaTilde = intersect(LambdaDelt, LambdaEst);
    OmL = Omega(LambdaTilde,:);
    %OmLPerp = eye(d) - pinv(OmL)*OmL;  the following condition implements this projection.
    if (size(OmL,1) < size(OmL,2))
        OmLPerp = eye(d) - OmL'*pinv(OmL*OmL')*OmL;
    else
        OmLPerp = eye(d) - pinv(OmL'*OmL)*(OmL'*OmL);
    end
    x_p = pinv( [OmLPerp*MM; OmL])*[OmLPerp*M'*y; zeros(length(LambdaTilde),1)];
    [LambdaEst, OmL] = find_rows_to_remove(x_p, Omega, cosparsity);
    %OmLPerp = eye(d) - pinv(OmL)*OmL;  the following condition implements this projection.
    if (size(OmL,1) < size(OmL,2))
        OmLPerp = eye(d) - OmL'*pinv(OmL*OmL')*OmL;
    else
        OmLPerp = eye(d) - pinv(OmL'*OmL)*(OmL'*OmL);
    end
    xhat = pinv( [OmLPerp*MM; OmL])*[OmLPerp*M'*y; zeros(length(LambdaEst),1)];
    LambdaEst = find(abs(Omega*xhat) < 1e-6);
    y_res = y - M*xhat;
    if nargin > 5 && isfield(options, 'x0')
        disp(['rel_err = ', num2str(norm(xhat-options.x0)/norm(x0))]);
        disp(['delta = ', num2str(norm(xhat-xprev)/norm(xhat))]);
        disp(['restricted isometry value = ', num2str((norm(M*(xhat-xprev))/norm(xhat-xprev))^2)]);
    end
    if norm(xhat-xprev)/norm(xhat) < convThresh || (norm(xhat) < convThresh)
        break;
    end
    xprev = xhat;
end
if nargin > 5 && isfield(options, 'D')
    xhat = options.D*xhat;
end
temp = zeros(p,1);
temp(LambdaEst) = 1;
LambdaEst =temp;
return;


function [LambdaEst, OmL] = find_rows_to_remove(x_p, Omega, cosparsity)
    p = size(Omega,1);
    l = cosparsity(1);
    [~, tempind] = sort(abs(Omega* x_p));
    LambdaEst = tempind(1:l);
    OmL = Omega(LambdaEst,:);
    return;