%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xhat, Lambdahat] = Analysis_IGP(y, Omega, M, cosparsity, funcs, params)

p = size(Omega,1);
m = size(M,1);
d = size(M,2);

L = funcs.projection_cosparsity(cosparsity, d, m, params.cosparsity_determination_parameter);
L_mu = funcs.cosparsity_for_gradient_step(cosparsity, L);

% if all d measurements are present, no need to run the algorithm
if m == d
    xhat = inv(M) * y;
    [temp, tempind] = sort(abs(Omega*xhat));
    Lambdahat = sort(tempind(1:cosparsity));
    return;
end

max_iter = params.max_iteration;
conv_thresh = params.convergence_threshold;
%give_up_thresh = 1e3*norm(pinv(M)*y,2);

xprev = zeros(d, 1);
LambdaEst = 1:p;

for j = 1:max_iter
    descent = M' * (y - M*xprev);
    if (strcmp(func2str(funcs.gradient_descent_step), 'gradient_descent_step_theoretical'))
        [mu, xhat, LambdaEst] = funcs.gradient_descent_step(xprev, descent, y, M, Omega, LambdaEst, L, funcs);
    else 
        LambdaEst = find(abs(Omega*xprev) < 1e-6);
        mu = funcs.gradient_descent_step(xprev, descent, y, M, Omega, LambdaEst, L_mu);
        %if j == 1
        %    disp(['mu=', num2str(mu)]);
        %end
        xhat_grad = xprev + mu * descent;

        [temp, tempind] = sort(abs(Omega*xhat_grad));
        LambdaEst = sort(tempind(1:L));
        xhat = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
    end 
    %if norm(xhat-xprev)/norm(xhat) < conv_thresh || norm(xhat) > give_up_thresh
    if norm(xhat-xprev)/norm(xhat) < conv_thresh
        break;
    end
    xprev = xhat;
end

xhat = funcs.finalize_estimate(xhat, LambdaEst, Omega, M, y);
temp = abs(Omega*xhat);
Lambdahat = sort(find(temp < params.zero_coefficient_tolerance));
return;
