%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [xhat, LambdaEst] = ACoSaMPM(y,Omega,M,l)

p = size(Omega,1);
m = size(M,1);
d = size(M,2);

max_iter = 500;
convThresh = 1e-7;
alpha = 0.001;

xprev = zeros(d, 1);
LambdaEst = 1:p;
y_res = y;
MM = M'*M;
for j = 1:max_iter
    if nargin > 4
        disp('*************')
        disp(['iteration ', num2str(j)]);
    end
    [~, tempind] = sort(abs(Omega*M'*y_res));
    LambdaDelt = sort(tempind(1:l));
    LambdaTilde = intersect(LambdaDelt, LambdaEst);
%    OmL = Omega(LambdaTilde,:);
%   OmLPerp = eye(d) - pinv(OmL)*OmL;
%    x_p = pinv( [OmLPerp*MM; OmL])*[OmLPerp*M'*y; zeros(length(LambdaTilde),1)];
    x_p = pinv( [M; alpha*Omega(LambdaTilde,:)])*[y; zeros(length(LambdaTilde),1)];
    [~, tempind] = sort(abs(Omega* x_p));
    LambdaEst = sort(tempind(1:l));
    xhat = x_p- pinv(Omega(LambdaEst,:)) * Omega(LambdaEst,:) * x_p;
    y_res = y - M*xhat;
    if nargin > 4
        disp(['rel_err = ', num2str(norm(xhat-x0)/norm(x0))]);
        disp(['delta = ', num2str(norm(xhat-xprev)/norm(xhat))]);
        disp(['restricted isometry value = ', num2str((norm(M*(xhat-xprev))/norm(xhat-xprev))^2)]);
    end
    if norm(xhat-xprev)/norm(xhat) < convThresh || (norm(xhat) < convThresh)
        break;
    end
    xprev = xhat;
end
temp = zeros(p,1);
temp(LambdaEst) = 1;
LambdaEst =temp;
return;

% function xhat = estimate(xin, OmL, M, y)
% xhat = xin - pinv(OmL) * OmL * xin;
% %[nr, nc] = size(OmL);
% %OmLPerp = eye(nc) - pinv(OmL)*OmL;
% %xhat = pinv([OmLPerp*M'*M; OmL]) * [OmLPerp*M'*y; zeros(nr, 1)];
 
