%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function r = compute_performance_stats_aa(configstr)
sd = 1;
rng(sd);
r = 0;
sigma =1.2; %sigma =2 for TV experiment
delta_interval = 0.05;
rho_interval = 0.05;
delta = delta_interval:delta_interval:1;
rho = rho_interval:rho_interval:1;
d = 120;
N = 50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% algorithms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
funcs1 = struct();
funcs1.projection_cosparsity = @projection_cosparsity_all;
funcs1.gradient_descent_step = @gradient_descent_step_static;
funcs1.projection = @projection_plain;
funcs1.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_highest;
funcs1.finalize_estimate = @finalize_estimate_do_nothing;

params1 = struct();
params1.cosparsity_determination_parameter = 1.0;
params1.max_iteration = 500;
params1.convergence_threshold = 1e-6;
params1.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_1 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs1, params1);
%%%%%%%%%%%%

funcs2 = struct();
funcs2.projection_cosparsity = @projection_cosparsity_ratio;
funcs2.gradient_descent_step = @gradient_descent_step_static;
funcs2.projection = @projection_plain;
funcs2.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs2.finalize_estimate = @finalize_estimate_do_nothing;

params2 = struct();
params2.cosparsity_determination_parameter = 0.75;
params2.max_iteration = 500;
params2.convergence_threshold = 1e-6;
params2.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_2 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs2, params2);

%%%%%%%%%%%

funcs3 = struct();
funcs3.projection_cosparsity = @projection_cosparsity_ratio;
funcs3.gradient_descent_step = @gradient_descent_step_static;
funcs3.projection = @projection_optimal_wrt_M;
funcs3.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs3.finalize_estimate = @finalize_estimate_do_nothing;

params3 = struct();
params3.cosparsity_determination_parameter = 0.75;
params3.max_iteration = 500;
params3.convergence_threshold = 1e-6;
params3.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_3 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs3, params3);

%%%%%%%%%%%

funcs5 = struct();
funcs5.projection_cosparsity = @projection_cosparsity_ratio;
funcs5.gradient_descent_step = @gradient_descent_step_theoretical_nonsense;
funcs5.projection = @projection_optimal_wrt_M;
funcs5.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs5.finalize_estimate = @finalize_estimate_do_nothing;

params5 = struct();
params5.cosparsity_determination_parameter = 0.75;
params5.max_iteration = 500;
params5.convergence_threshold = 1e-6;
params5.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_5 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs5, params5);

%%%%%%%%%%%

funcs6 = struct();
funcs6.projection_cosparsity = @projection_cosparsity_ratio;
funcs6.gradient_descent_step = @gradient_descent_step_minimize_M;
funcs6.projection = @projection_optimal_wrt_M;
funcs6.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs6.finalize_estimate = @finalize_estimate_do_nothing;

params6 = struct();
params6.cosparsity_determination_parameter = 0.75;
params6.max_iteration = 500;
params6.convergence_threshold = 1e-6;
params6.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_6 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs6, params6);


%%%%%%%%%%%

funcs7 = struct();
funcs7.projection_cosparsity = @projection_cosparsity_ratio;
funcs7.gradient_descent_step = @gradient_descent_step_theoretical;
funcs7.projection = @projection_optimal_wrt_M;
funcs7.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs7.finalize_estimate = @finalize_estimate_do_nothing;

params7 = struct();
params7.cosparsity_determination_parameter = 0.75;
params7.max_iteration = 500;
params7.convergence_threshold = 1e-6;
params7.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_7 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs7, params7);


%%%%%%%%%%%

funcs8 = struct();
funcs8.projection_cosparsity = @projection_cosparsity_ratio;
funcs8.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs8.projection = @projection_optimal_wrt_M;
funcs8.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs8.finalize_estimate = @finalize_estimate_do_nothing;

params8 = struct();
params8.cosparsity_determination_parameter = 0.75;
params8.max_iteration = 500;
params8.convergence_threshold = 1e-6;
params8.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_8 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs8, params8);


%%%%%%%%%%%

funcs9 = struct();
funcs9.projection_cosparsity = @projection_cosparsity_all;
funcs9.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs9.projection = @projection_optimal_wrt_M;
funcs9.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs9.finalize_estimate = @finalize_estimate_do_nothing;

params9 = struct();
params9.cosparsity_determination_parameter = 1.0;
params9.max_iteration = 500;
params9.convergence_threshold = 1e-6;
params9.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_9 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs9, params9);



%%%%%%%%%%%

funcs10 = struct();
funcs10.projection_cosparsity = @projection_cosparsity_all;
funcs10.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs10.projection = @projection_plain;
funcs10.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs10.finalize_estimate = @finalize_estimate_do_nothing;

params10 = struct();
params10.cosparsity_determination_parameter = 1.0;
params10.max_iteration = 500;
params10.convergence_threshold = 1e-6;
params10.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_10 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs10, params11);


%%%%%%%%%%%

funcs11 = struct();
funcs11.projection_cosparsity = @projection_cosparsity_all;
funcs11.gradient_descent_step = @gradient_descent_step_theoretical;
funcs11.projection = @projection_optimal_wrt_M;
funcs11.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs11.finalize_estimate = @finalize_estimate_do_nothing;

params11 = struct();
params11.cosparsity_determination_parameter = 1.0;
params11.max_iteration = 500;
params11.convergence_threshold = 1e-6;
params11.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_11 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs11, params11);



%%%%%%%%%%%

funcs12 = struct();
funcs12.projection_cosparsity = @projection_cosparsity_all;
funcs12.gradient_descent_step = @gradient_descent_step_theoretical;
funcs12.projection = @projection_plain;
funcs12.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs12.finalize_estimate = @finalize_estimate_do_nothing;

params12 = struct();
params12.cosparsity_determination_parameter = 1.0;
params12.max_iteration = 500;
params12.convergence_threshold = 1e-6;
params12.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_12 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs12, params12);


%%%%%%%%%%%

funcs13 = struct();
funcs13.projection_cosparsity = @projection_cosparsity_all;
funcs13.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs13.projection = @projection_suboptimal_wrt_M;
funcs13.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs13.finalize_estimate = @finalize_estimate_do_nothing;

params13 = struct();
params13.cosparsity_determination_parameter = 1.0;
params13.max_iteration = 500;
params13.convergence_threshold = 1e-6;
params13.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_13 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs13, params13);


%%%%%%%%%%%


funcs14 = struct();
funcs14.projection_cosparsity = @projection_cosparsity_ratio;
funcs14.gradient_descent_step = @gradient_descent_step_theoretical;
funcs14.projection = @projection_plain;
funcs14.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs14.finalize_estimate = @finalize_estimate_do_nothing;

params14 = struct();
params14.cosparsity_determination_parameter = 0.75;
params14.max_iteration = 500;
params14.convergence_threshold = 1e-6;
params14.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_14 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs14, params14);

%%%%%%%%%%%
% HTP with adaptive step size

funcs15 = struct();
funcs15.projection_cosparsity = @projection_cosparsity_best;
funcs15.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs15.projection = @projection_optimal_wrt_M;
funcs15.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs15.finalize_estimate = @finalize_estimate_do_nothing;

params15 = struct();
params15.cosparsity_determination_parameter = 1.0;
params15.max_iteration = 500;
params15.convergence_threshold = 1e-6;
params15.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_15 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs15, params15);


%%%%%%%%%%%
% IHT with adaptive step size

funcs16 = struct();
funcs16.projection_cosparsity = @projection_cosparsity_best;
funcs16.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs16.projection = @projection_plain;
funcs16.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs16.finalize_estimate = @finalize_estimate_pinv;

params16 = struct();
params16.cosparsity_determination_parameter = 1.0;
params16.max_iteration = 500;
params16.convergence_threshold = 1e-6;
params16.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_16 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs16, params16);


%%%%%%%%%%%
% HTP with static step size

funcs17 = struct();
funcs17.projection_cosparsity = @projection_cosparsity_best;
funcs17.gradient_descent_step = @gradient_descent_step_static;
funcs17.projection = @projection_optimal_wrt_M;
funcs17.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs17.finalize_estimate = @finalize_estimate_do_nothing;

params17 = struct();
params17.cosparsity_determination_parameter = 1.0;
params17.max_iteration = 500;
params17.convergence_threshold = 1e-6;
params17.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_17 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs17, params17);


%%%%%%%%%%%
% IHT with static step size

funcs18 = struct();
funcs18.projection_cosparsity = @projection_cosparsity_best;
funcs18.gradient_descent_step = @gradient_descent_step_static;
funcs18.projection = @projection_plain;
funcs18.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs18.finalize_estimate = @finalize_estimate_pinv;

params18 = struct();
params18.cosparsity_determination_parameter = 1.0;
params18.max_iteration = 500;
params18.convergence_threshold = 1e-6;
params18.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_18 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs18, params18);

%%%%%%%%%%%

funcs19 = struct();
funcs19.projection_cosparsity = @projection_cosparsity_best;
funcs19.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs19.projection = @projection_suboptimal_wrt_M;
funcs19.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs19.finalize_estimate = @finalize_estimate_do_nothing;

params19 = struct();
params19.cosparsity_determination_parameter = 1.0;
params19.max_iteration = 500;
params19.convergence_threshold = 1e-6;
params19.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_19 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs19, params19);


%%%%%%%%%%%

funcs20 = struct();
funcs20.projection_cosparsity = @projection_cosparsity_best;
funcs20.gradient_descent_step = @gradient_descent_step_theoretical_adaptive;
funcs20.projection = @projection_suboptimal_wrt_M2;
funcs20.cosparsity_for_gradient_step = @cosparsity_for_gradient_step_lowest;
funcs20.finalize_estimate = @finalize_estimate_do_nothing;

params20 = struct();
params20.cosparsity_determination_parameter = 1.0;
params20.max_iteration = 500;
params20.convergence_threshold = 1e-6;
params20.zero_coefficient_tolerance = 1e-6;

Analysis_IHT_20 = @(y, Omega, M, l) Analysis_IGP(y, Omega, M, l, funcs20, params20);


%%%%%%%%%%%

ACoSaMP_bestl = @(y, Omega, M, l) ACoSaMP(y, Omega, M, min(size(M,2) - size(M,1)/2, l),1); %Using the lowest cosparsity that guarantees uniqueness
ACoSaMP_th_l = @(y, Omega, M, l) ACoSaMP(y, Omega, M, l, (2*l-size(Omega,1))./max(l,1)); %Using ACoSaMP with a = (2l-p)/l.
ACoSaMP_th_bestl = @(y, Omega, M, l) ACoSaMP(y, Omega, M, min(size(M,2) - size(M,1)/2, l), (2*l-size(Omega,1))/min(size(M,2) - size(M,1)/2, l)); %Using the lowest cosparsity that guarantees uniqueness for both projection steps in ACoSaMP
ACoSaMP_l = @(y, Omega, M, l) ACoSaMP(y, Omega, M, l, 1); %Using the regular cosparsity
ASP_l = @(y, Omega, M, l) ASP(y, Omega, M, l, 1);  %Using the regular cosparsity
ASP_th_l = @(y, Omega, M, l) ASP(y, Omega, M, l, (2*l-size(Omega,1))./max(l,1)); %Using ASP with a = (2l-p)/l.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% end algorithms
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%algos = {Analysis_IHT_1, Analysis_IHT_2, Analysis_IHT_3, Analysis_IHT_4, Analysis_IHT_5, Analysis_IHT_6};
%algos = {Analysis_IHT_8, Analysis_IHT_9, Analysis_IHT_11};
%algos = {Analysis_IHT_mem2_8, Analysis_IHT_mem2_9, Analysis_IHT_mem2_11};
%algos = {Analysis_IHT_mem2_8, Analysis_IHT_8,  @Analysis_L1, @Analysis_GAP};
%algos = {Analysis_IHT_7};
%algos = {Analysis_IHT_mem_12, Analysis_IHT_mem_13};
%algos = {@Analysis_L1, @Analysis_GAP};
%algos = {ACoSaMP_th_bestl}
%algos = {ACoSaMPM_bestl, @Analysis_GAP, Analysis_IHT_19, Analysis_IHT_20}
%algos = {ACoSaMP_l, ASP_l, ACoSaMP_th}
algos = {Analysis_IHT_16, Analysis_IHT_15,Analysis_IHT_18, Analysis_IHT_17,ASP_l,ACoSaMP_l,ASP_th_l,ACoSaMP_th_l,@Analysis_L1, @Analysis_GAP};
evals = {@Evaluate_Error};
tol_perfect_recovery = 1e-4;
na = length(algos);
nd = length(delta);
nr = length(rho);

configint = str2double(configstr);
[itd, itr] = getInds([nd, nr], configint);  % only because ind2sub fails on igrida

if itr > nr
    return;
end
if itd > nd
    return;
end

p = floor(sigma*d + 0.5)
m = floor(delta(itd)*d + 0.5)
l = min(d-floor(rho(itr)*m + 0.5), d-1)

results = Run_Analysis_Experiments(N, d, p, m, l, algos, evals,'random');
%results = squeeze(results);

rec_vals = zeros(na, N);
rec_rate = zeros(na, 1);
rel_err = ones(na, 1);
for ita = 1:na
    rec_vals(ita,:) = results(:, 1, ita);
    rec_rate(ita) = length(find(results(:, 1, ita) < tol_perfect_recovery)) / N;
    rel_err(ita) = mean(results(:, 1, ita));
end


r = rec_rate(1);
outputfilename = sprintf('OUTPUTDIR/analysisAlgos%s.mat', configstr);
save(outputfilename, 'rec_vals','rec_rate', 'rel_err');

return;

function [I, J] = getInds(siz, ii)
J = ceil(ii/siz(1));
I = ii - siz(1)*(J-1);
