%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                        
% Analysis Greedy Like Algorithms (AGLA)
% Version 1.0                                    
%     
% Copyright 2012 Raja Giryes, Sangnam Nam, Michael Elad, Remi Gribonval, and Mike. E. Davies
% 
% For all details please refer to README.TXT
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose.                                        
%                                                                                                                                                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [mu, xhat, LambdaEst] = gradient_descent_step_theoretical(xprev, descent, y, M, Omega, LambdaEst, cosparsity, funcs)
L = cosparsity;
L_mu = cosparsity;
mu = gradient_descent_step_theoretical_nonsense(xprev, descent, y, M, Omega, LambdaEst, L_mu);
xhat_grad = xprev + mu * descent;
[~, tempind] = sort(abs(Omega*xhat_grad));
LambdaEst = sort(tempind(1:L));
xhat_nonsense = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
mu_nonsense = mu;
error_nonsens = norm(y - M*xhat_nonsense).^2;
LambdaEstAll = LambdaEst;

mu = gradient_descent_step_static(xprev, descent, y, M, Omega, LambdaEst, L_mu);
xhat_grad = xprev + mu * descent;
[~, tempind] = sort(abs(Omega*xhat_grad));
LambdaEst = sort(tempind(1:L));
xhat_static = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
mu_static = mu;
error_static = norm(y - M*xhat_nonsense).^2;
LambdaEstAll = [LambdaEstAll,LambdaEst];
mu = gradient_descent_step_theoretical_nonsense(xprev, descent, y, M, Omega, LambdaEst, L_mu);
xhat_grad = xprev + mu * descent;
[~, tempind] = sort(abs(Omega*xhat_grad));
LambdaEst = sort(tempind(1:L));
xhat_static_opt = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
mu_static_opt = mu;
error_static_opt = norm(y - M*xhat_nonsense).^2;
LambdaEstAll = [LambdaEstAll,LambdaEst];


mu = gradient_descent_step_minimize_M(xprev, descent, y, M, Omega, LambdaEst, L_mu);
xhat_grad = xprev + mu * descent;
[~, tempind] = sort(abs(Omega*xhat_grad));
LambdaEst = sort(tempind(1:L));
xhat_minimizeM = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
mu_minimizeM = mu;
error_minimizeM = norm(y - M*xhat_nonsense).^2;
LambdaEstAll = [LambdaEstAll,LambdaEst];
mu = gradient_descent_step_theoretical_nonsense(xprev, descent, y, M, Omega, LambdaEst, L_mu);
xhat_grad = xprev + mu * descent;
[~, tempind] = sort(abs(Omega*xhat_grad));
LambdaEst = sort(tempind(1:L));
xhat_minimizeM_opt = funcs.projection(xhat_grad, Omega(LambdaEst,:), M, y);
mu_minimizeM_opt = mu;
error_minimizeM_opt = norm(y - M*xhat_nonsense).^2;
LambdaEstAll = [LambdaEstAll,LambdaEst];


x_hat = [xhat_nonsense, xhat_static, xhat_static_opt, xhat_minimizeM, xhat_minimizeM_opt];
mu = [mu_nonsense, mu_static, mu_static_opt, mu_minimizeM, mu_minimizeM_opt];
[~,ind] = min([error_nonsens, error_static, error_static_opt, error_minimizeM, error_minimizeM_opt]);
xhat = x_hat(:,ind);
mu = mu(ind);
LambdaEst = LambdaEstAll(:,ind);